<SendDirectToMachineAttribute(DirectSendControl.Mach3)> _
Public Class MachineDefinition
    Inherits MachineDefinitionBase
    Implements IMachineDefinition_2_1
	
	''' <summary>
	''' The version of this machine definition as shown by choosing the Version command from the Help menu of the Configure Machine Definition dialog
	''' </summary>
	Private version As System.Version =  New System.Version("1.4.5")

    ''' <summary>
    ''' Local variable to store the format for the number of decimal places after the point.
    ''' </summary>
    Private format As String
	
	''' <summary>
	''' Local variable that is used to store the configuration.
	''' </summary>
	''' <remarks>Any changes made by the user from the 'Configure Machine Definition' command are assigned to this variable.</remarks>
	Private _configuration As Configuration
	
	''' <summary>
    ''' Local variable to store tool length offset as set in the Tool Manger 'Offset' field
    ''' </summary>
	Private toolLengthOffset As Double

#Region "Events"

	''' <summary>
	''' This event handler is invoked when this Machine Definition has finished compiling.
	''' </summary>
	''' <param name="e">Provides data for the compiled event.</param>
    ''' <remarks>Here is where you should perform any one time initialization.</remarks>
	Private Sub Complied(ByVal e As CompiledEventArgs) Implements IMachineDefinition_2_1.Compiled

		UserConfiguredInputDialog.DeleteAllInputBoxes()
        UserConfiguredInputDialog.AddInputBox("Program Number", 50, "1")
        UserConfiguredInputDialog.AddInputBox("Fixture Offset", 50, "54")
		
	End Sub
 
    ''' <summary>
    ''' This event handler is invoked first when the Nc code is generated and is where you
	''' configure the general settings.
    ''' </summary>
    ''' <param name="e">Provides data for the general settings event.</param>
    Private Sub GeneralSettings(ByVal e As GeneralSettingsEventArgs) Implements IMachineDefinition_2_1.GeneralSettings

        e.OutputLineNumbers = _configuration.OutputLineNumbers
        e.StartLineNumber = 10
        e.LineNumberIncrement = 10
        e.ArcBreak = ArcBreak.DoNotBreak
		e.NcCodeExtension = "tap"
		e.ChipToChipAtcTime = _configuration.ChipToChipAtcTime
		e.RapidPositionAccelerationDeceleration = _configuration.RapidPositionAccelerationDeceleration
		e.SubRoutinesAtStart = False	
		e.SubRoutinesStartNumber = 1000
		e.SubRoutinesNumberIncrement = 10
		e.LineNumbering = LineNumbering.NoneSequentially
		
        'Metric parts need 3 decimal places after the point, imperial 4
        If e.Metric Then
            format = "0.0##"
            e.MinimumArcLength = 0.005
            e.MinimumArcRadius = 0.005
            e.MaximumArcRadius = 10000
        Else
            format = "0.0###"
            e.MinimumArcLength = 0.0002
            e.MinimumArcRadius = 0.0002
            e.MaximumArcRadius = 400
       		e.RapidTraverseRate = _configuration.RapidTraverseRate / 25.4
		End If

        programNumber = UserConfiguredInputDialog.GetInputBoxValue("Program Number")
        fixtureOffset = UserConfiguredInputDialog.GetInputBoxValue("Fixture Offset")

    End Sub

    ''' <summary>
    ''' This event handler is invoked so the start of the file can be created.
    ''' </summary>
    ''' <param name="e">Provides data for the program start event.</param>
    ''' <remarks>Here is where you should create the begining of the file, for example the % sign as used by a Fanuc.</remarks>
    Private Sub FileStart(ByVal e As FileStartEventArgs) Implements IMachineDefinition_2_1.FileStart
        
		NcCode.NewString("%")
        NcCode.NewLine()

    End Sub

    ''' <summary>
    ''' This event handler is invoked so the start of the program can be created.
    ''' </summary>
    ''' <param name="e">Provides data for the program start event.</param>
    ''' <remarks>Here is where you should create any preparatory code.</remarks>
    Private Sub ProgramStart(ByVal e As ProgramStartEventArgs) Implements IMachineDefinition_2_1.ProgramStart

        'Output the program number and the name of the part
		If _configuration.CommentsType = CommentsType.Parenthesis Then
    		NcCode.NewString("O" & programNumber & " (" & e.SharpCamFilename.ToUpper() & ")")
		Else
    		NcCode.NewString("O" & programNumber & " //" & e.SharpCamFilename.ToUpper())
		End If
        NcCode.NewLine()
        
		If _configuration.OutputCycleTime Then
        	NcCode.IncrementLineNumber()
			If _configuration.CommentsType = CommentsType.Parenthesis Then
    			NcCode.NewString("(CYCLE TIME " & New System.TimeSpan( 0, 0, e.CycleTime).TotalMinutes.ToString("0.00") & " MINUTES)")
			Else
    			NcCode.NewString("//CYCLE TIME " & New System.TimeSpan( 0, 0, e.CycleTime).TotalMinutes.ToString("0.00") & " MINUTES")
			End If			
        	NcCode.NewLine()
		End If

		'List all tools used in this program
        If e.ToolNames.Count > 0 Then
            For Each name As String In e.ToolNames
                NcCode.IncrementLineNumber()
				If _configuration.CommentsType = CommentsType.Parenthesis Then
                	NcCode.NewString("(" & name.ToUpper() & ")")
				Else
                	NcCode.NewString("//" & name.ToUpper())
				End If
                NcCode.NewLine()
            Next
        End If

        NcCode.LineNumber("N")
		Dim units, arcCentres As String
		If e.Metric Then
			units = "G21"
		Else
			units = "G20"
		End If
		If _configuration.ArcCentre = ArcCentre.Incremental Then
			arcCentres = "G91.1"
		Else
			arcCentres = "G90.1"
		End If
        NcCode.NewString(units & " G40 G17 G80 " & arcCentres)			
        NcCode.NewLine()
		
        Me.PositionForToolChange()
		
		If _configuration.OutputM01ForATC = M01Type.OutputBeforeATC
        	NcCode.LineNumber("N")
        	NcCode.NewString("M01")
			NcCode.NewLine()
		End If
	
    End Sub

    ''' <summary>
    ''' This event handler is invoked for every tool change.
    ''' </summary>
    ''' <param name="e">Provides data for the tool change event.</param>
    Private Sub ToolChange(ByVal e As ToolChangeEventArgs) Implements IMachineDefinition_2_1.ToolChange
		
		If _configuration.OutputLineNumberAtATC And Not _configuration.OutputLineNumbers Then
			NcCode.NewString("N" & e.ToolNumber & " ")
		End If
		If _configuration.ToolChangerType = ToolChangerType.NoPreSelectType
       		NcCode.LineNumber("N")
			If _configuration.CommentsType = CommentsType.Parenthesis Then
        		NcCode.NewString("T" & e.ToolNumber & " M6 (" & e.ToolName.ToUpper() & ")")			
			Else
        		NcCode.NewString("T" & e.ToolNumber & " M6 //" & e.ToolName.ToUpper())			
			End If
        	NcCode.NewLine()
		Else If _configuration.ToolChangerType = ToolChangerType.UseMacroMCode
			NcCode.LineNumber("N")
			If _configuration.CommentsType = CommentsType.Parenthesis Then
        		NcCode.NewString("M" & _configuration.MCodeForAtc & " P" & e.ToolNumber & " (" & e.ToolName.ToUpper() & ")")
        	Else
        		NcCode.NewString("M" & _configuration.MCodeForAtc & " P" & e.ToolNumber & " //" & e.ToolName.ToUpper())
			End If
			NcCode.NewLine()
			NcCode.LineNumber("N")
        	NcCode.NewString("M6")
        	NcCode.NewLine()
		Else If _configuration.ToolChangerType = ToolChangerType.PreSelectType
       		NcCode.LineNumber("N")
			If _configuration.CommentsType = CommentsType.Parenthesis Then
				NcCode.NewString("T" & e.ToolNumber & " M6 (" & e.ToolName.ToUpper() & ")")
			Else
				NcCode.NewString("T" & e.ToolNumber & " M6 //" & e.ToolName.ToUpper())
			End If
        	NcCode.NewLine()
		End If
		If _configuration.OutputM01ForATC = M01Type.OutputAfterATC
        	NcCode.LineNumber("N")
        	NcCode.NewString("M01")
	        NcCode.NewLine()
		End If
	
		If _configuration.AddOffsetValue Then
			toolLengthOffset = e.ToolOffsetNumber
		Else
			toolLengthOffset = 0
		End If
	
    End Sub

    ''' <summary>
    ''' This event handler is invoked before every tool change, unless it is the first, where it is not called.
    ''' </summary>
    ''' <param name="e">Provides data for the cancel tool change event.</param>
    Private Sub CancelToolChange(ByVal e As CancelToolChangeEventArgs) Implements IMachineDefinition_2_1.CancelToolChange
		
		Me.PositionForToolChange()
		
		If _configuration.OutputM01ForATC = M01Type.OutputBeforeATC
        	NcCode.LineNumber("N")
        	NcCode.NewString("M01")
			NcCode.NewLine()
		End If

    End Sub

    ''' <summary>
    ''' This event handler is invoked for the first position after a tool change for Profiling and Pocketing operations.
    ''' </summary>
    ''' <param name="e">Provides data for the first position after tool change event.</param>
    Private Sub FirstPositionAfterToolChange(ByVal e As FirstPositionAfterToolChangeEventArgs) Implements IMachineDefinition_2_1.FirstPositionAfterToolChange

		NcCode.LineNumber("N")

    	NcCode.NewWord("G0", Modal.Off)	
		If _configuration.OutputFixtureOffset Then
			NcCode.NewString("G90 G" & fixtureOffset & " ")			
		Else
			NcCode.NewString("G90 ")			
		End If			
    	NcCode.NewDimensionedWord("X", e.PositionX.ToString(format, CultureInfo), Modal.Off)
    	NcCode.NewDimensionedWord("Y", e.PositionY.ToString(format, CultureInfo), Modal.Off)
    	NcCode.NewString(e.ToolDirection & " S" & e.SpindleSpeed)
		
        If _configuration.ToolChangerType = ToolChangerType.PreSelectType And e.ToolNumbers.Count > 1
        	NcCode.NewString(" T" & e.NextToolNumber)
		End If
        NcCode.NewLine()

		NcCode.LineNumber("N")
		If _configuration.CodeToApplyToolOffsets = CodeToApplyToolOffsets.G43 Then
        	NcCode.NewString("G43 ")
		Else
        	NcCode.NewString("G44 ")
		End If
        NcCode.NewDimensionedWord("Z", (e.PositionZ + toolLengthOffset).ToString(format, CultureInfo), Modal.Off)
		NcCode.NewString("H" & e.ToolNumber & " " & e.Coolant)	
        NcCode.NewLine()

    End Sub

    ''' <summary>
    ''' This event handler is invoked for all Profiling and Pocketing rapid positions, other than
    ''' the <see cref="FirstPositionAfterToolChange"/>.
    ''' </summary>
    ''' <param name="e">Provides data for the rapid position event.</param>
    Private Sub RapidPositiong(ByVal e As RapidPositionEventArgs) Implements IMachineDefinition_2_1.RapidPositioning

        NcCode.LineNumber("N")
        NcCode.NewWord("G0", Modal.On)
        NcCode.NewDimensionedWord("X", e.AbsEndX.ToString(format, CultureInfo), Modal.On)
        NcCode.NewDimensionedWord("Y", e.AbsEndY.ToString(format, CultureInfo), Modal.On)
        NcCode.NewDimensionedWord("Z", (e.AbsZ + toolLengthOffset).ToString(format, CultureInfo), Modal.On)
        NcCode.NewLine()

    End Sub

    ''' <summary>
    ''' This event handler is invoked for every linear interpolation move.
    ''' </summary>
    ''' <param name="e">Provides data for the linear interpolation event.</param>
    Private Sub LinearInterpolation(ByVal e As LinearInterpEventArgs) Implements IMachineDefinition_2_1.LinearInterpolation

        NcCode.LineNumber("N")
        NcCode.NewWord("G1", Modal.On)
        NcCode.NewDimensionedWord("X", e.AbsEndX.ToString(format, CultureInfo), Modal.On)
        NcCode.NewDimensionedWord("Y", e.AbsEndY.ToString(format, CultureInfo), Modal.On)
        NcCode.NewDimensionedWord("Z", (e.AbsEndZ + toolLengthOffset).ToString(format, CultureInfo), Modal.On)
        NcCode.NewDimensionedWord("F", e.FeedRate.ToString(format, CultureInfo), Modal.On)
        NcCode.NewLine()

    End Sub

    ''' <summary>
    ''' This event handler is invoked when Cutter Radius Compensation (CRC) is applied on the left.
    ''' </summary>
    ''' <param name="e">Provides data for the apply CRC left event.</param>
    ''' <remarks>This is a linear move. CRC is not applied on circular interpolation moves.</remarks>
    Private Sub ApplyCRCLeft(ByVal e As ApplyCRCLeftEventArgs) Implements IMachineDefinition_2_1.ApplyCRCLeft

        NcCode.LineNumber("N")
        NcCode.NewWord("G1", Modal.On)
        NcCode.NewString("G41 ")
        NcCode.NewDimensionedWord("X", e.AbsEndX.ToString(format, CultureInfo), Modal.On)
        NcCode.NewDimensionedWord("Y", e.AbsEndY.ToString(format, CultureInfo), Modal.On)
        NcCode.NewDimensionedWord("Z", (e.AbsEndZ + toolLengthOffset).ToString(format, CultureInfo), Modal.On)
        NcCode.NewDimensionedWord("F", e.FeedRate.ToString(format, CultureInfo), Modal.On)
        NcCode.NewLine()

    End Sub

    ''' <summary>
    ''' This event handler is invoked when Cutter Radius Compensation (CRC) is applied on the right.
    ''' </summary>
    ''' <param name="e">Provides data for the apply CRC right event</param>
    ''' <remarks>This is a linear move. CRC is not applied on circular interpolation moves</remarks>
    Private Sub ApplyCRCRight(ByVal e As ApplyCRCRightEventArgs) Implements IMachineDefinition_2_1.ApplyCRCRight

        NcCode.LineNumber("N")
        NcCode.NewWord("G1", Modal.On)
        NcCode.NewString("G42 ")
        NcCode.NewDimensionedWord("X", e.AbsEndX.ToString(format, CultureInfo), Modal.On)
        NcCode.NewDimensionedWord("Y", e.AbsEndY.ToString(format, CultureInfo), Modal.On)
        NcCode.NewDimensionedWord("Z", (e.AbsEndZ + toolLengthOffset).ToString(format, CultureInfo), Modal.On)
        NcCode.NewDimensionedWord("F", e.FeedRate.ToString(format, CultureInfo), Modal.On)
        NcCode.NewLine()

    End Sub

    ''' <summary>
    ''' This event handler is invoked when Cutter Radius Compensation (CRC) is canceled.
    ''' </summary>
    ''' <param name="e">Provides data for the cancel CRC right event.</param>
    ''' <remarks>This is a linear move. CRC is not cancelled on circular interpolation moves.</remarks>
    Private Sub CancelCRC(ByVal e As CancelCRCEventArgs) Implements IMachineDefinition_2_1.CancelCRC

        NcCode.LineNumber("N")
        NcCode.NewWord("G1", Modal.On)
        NcCode.NewString("G40 ")
        NcCode.NewDimensionedWord("X", e.AbsEndX.ToString(format, CultureInfo), Modal.On)
        NcCode.NewDimensionedWord("Y", e.AbsEndY.ToString(format, CultureInfo), Modal.On)
        NcCode.NewDimensionedWord("Z", (e.AbsEndZ + toolLengthOffset).ToString(format, CultureInfo), Modal.On)
        NcCode.NewDimensionedWord("F", e.FeedRate.ToString(format, CultureInfo), Modal.On)
        NcCode.NewLine()

    End Sub

    ''' <summary>
    ''' This event handler is invoked for every counter clockwise circular interpolation move.
    ''' </summary>
    ''' <param name="e">Provides data for the clockwise circular interpolation event</param>
    Private Sub CWInterpolation(ByVal e As CWInterpolationEventArgs) Implements IMachineDefinition_2_1.CWInterpolation

        NcCode.LineNumber("N")
        NcCode.NewWord("G2", Modal.On)
        NcCode.NewDimensionedWord("X", e.AbsEndX.ToString(format, CultureInfo), Modal.On)
        NcCode.NewDimensionedWord("Y", e.AbsEndY.ToString(format, CultureInfo), Modal.On)
        NcCode.NewDimensionedWord("Z", (e.AbsEndZ + toolLengthOffset).ToString(format, CultureInfo), Modal.On)
		If _configuration.ArcCentre = ArcCentre.Incremental Then
        	NcCode.NewDimensionedWord("I", e.IncrementalCentreX.ToString(format, CultureInfo), Modal.On)
        	NcCode.NewDimensionedWord("J", e.IncrementalCentreY.ToString(format, CultureInfo), Modal.On)	
		Else
        	NcCode.NewString("I" & e.AbsCentreX.ToString(format, CultureInfo))
        	NcCode.NewString(" J" & e.AbsCentreY.ToString(format, CultureInfo))	
		End If
        NcCode.NewDimensionedWord("F", e.FeedRate.ToString(format, CultureInfo), Modal.On)
        NcCode.NewLine()

    End Sub

    ''' <summary>
    ''' This event handler is invoked for every clockwise circular interpolation move
    ''' </summary>
    ''' <param name="e">Provides data for the circular interpolation event.</param>
    Private Sub CCWInterpolation(ByVal e As CCWInterpolationEventArgs) Implements IMachineDefinition_2_1.CCWInterpolation

        NcCode.LineNumber("N")
        NcCode.NewWord("G3", Modal.On)
        NcCode.NewDimensionedWord("X", e.AbsEndX.ToString(format, CultureInfo), Modal.On)
        NcCode.NewDimensionedWord("Y", e.AbsEndY.ToString(format, CultureInfo), Modal.On)
        NcCode.NewDimensionedWord("Z", (e.AbsEndZ + toolLengthOffset).ToString(format, CultureInfo), Modal.On)
		If _configuration.ArcCentre = ArcCentre.Incremental Then
        	NcCode.NewDimensionedWord("I", e.IncrementalCentreX.ToString(format, CultureInfo), Modal.On)
        	NcCode.NewDimensionedWord("J", e.IncrementalCentreY.ToString(format, CultureInfo), Modal.On)	
		Else
        	NcCode.NewString("I" & e.AbsCentreX.ToString(format, CultureInfo))
        	NcCode.NewString(" J" & e.AbsCentreY.ToString(format, CultureInfo))	
		End If
        NcCode.NewDimensionedWord("F", e.FeedRate.ToString(format, CultureInfo), Modal.On)
        NcCode.NewLine()

    End Sub

    ''' <summary>
    ''' This event handler is invoked to drill the first hole after a tool change.
    ''' </summary>
    ''' <param name="e">Provides data for the drill first hole after toolchange event</param>
    Private Sub DrillFirstHoleAfterToolChange(ByVal e As DrillFirstHoleAfterToolChangeEventArgs) Implements IMachineDefinition_2_1.DrillFirstHoleAfterToolChange

        If e.DrillingType = DrillingType.RigidTapping Then
            MessageBox.Show("Rigid tapping not supported on the Mach3 control", "Mach3", MessageBoxButtons.OK, MessageBoxIcon.Warning)
			Return
        End If
		NcCode.LineNumber("N")		
		NcCode.NewWord("G0", Modal.Off)
		If _configuration.OutputFixtureOffset Then
			NcCode.NewString("G90 G" & fixtureOffset & " ")			
		Else
			NcCode.NewString("G90 ")			
		End If
		NcCode.NewDimensionedWord("X", e.PositionX.ToString(format, CultureInfo), Modal.Off)
		NcCode.NewDimensionedWord("Y", e.PositionY.ToString(format, CultureInfo), Modal.Off)
		NcCode.NewString(e.ToolDirection & " S" & e.SpindleSpeed)
		NcCode.NewLine()
		NcCode.LineNumber("N")
		If _configuration.CodeToApplyToolOffsets = CodeToApplyToolOffsets.G43 Then
			NcCode.NewString("G43 ")
		Else
			NcCode.NewString("G44 ")
		End If
		NcCode.NewDimensionedWord("Z", (e.IntialRapid + toolLengthOffset).ToString(format, CultureInfo), Modal.Off)
		NcCode.NewString("H" & e.ToolNumber & " " & e.Coolant)
		NcCode.NewLine()

        NcCode.LineNumber("N")
        Dim feedrate As Double
        If e.DrillingType = DrillingType.Drilling And e.Dwell = 0 Then
            NcCode.NewString("G81 ")
            feedrate = e.FeedRate
        ElseIf e.DrillingType = DrillingType.Drilling And e.Dwell > 0 Then
            NcCode.NewString("G82 ")
            feedrate = e.FeedRate
        ElseIf e.DrillingType = DrillingType.PeckDrill Then
            NcCode.NewString("G83 ")
            feedrate = e.FeedRate
        ElseIf e.DrillingType = DrillingType.ChipBreaker Then
            NcCode.NewString("G73 ")
            feedrate = e.FeedRate
        ElseIf e.DrillingType = DrillingType.Tapping Then
            NcCode.NewString("G84 ")
            feedrate = e.Pitch * e.SpindleSpeed * 0.95 'Reduce feedrate by 5% for floating holder
        ElseIf e.DrillingType = DrillingType.Boring Or e.DrillingType = DrillingType.Reaming Then
            NcCode.NewString("G85 ")
            feedrate = e.FeedRate
        ElseIf e.DrillingType = DrillingType.CustomCycle1 Then
			'Add code for additional drilling cycle
		End If
        Dim returnType As String
        If e.RetractToIntialRapid Then
            returnType = "G98"
        Else
            returnType = "G99"
        End If
        NcCode.NewString(returnType & " R" & (e.FeedFrom + toolLengthOffset).ToString(format, CultureInfo) & " ")
        NcCode.NewDimensionedWord("Z", (e.FinishDepth + toolLengthOffset).ToString(format, CultureInfo), Modal.Off)
        If (e.DrillingType = DrillingType.Drilling) And e.Dwell > 0 Then
            NcCode.NewString("P" & e.Dwell * 1000 & " ")
        ElseIf e.DrillingType = DrillingType.PeckDrill Or e.DrillingType = DrillingType.ChipBreaker Then
            NcCode.NewString("Q" & e.PeckAmount.ToString(format, CultureInfo) & " ")
        End If
        NcCode.NewDimensionedWord("F", feedrate.ToString(format, CultureInfo), Modal.Off)
        NcCode.NewLine()

    End Sub

    ''' <summary>
    ''' This event handler is invoked to drill the first hole in an operation that is NOT after a Tool change.
    ''' </summary>
    ''' <param name="e">Provides data for the drill first hole in operation event.</param>
    Private Sub DrillFirstHoleInOperation(ByVal e As DrillFirstHoleInOperationEventArgs) Implements IMachineDefinition_2_1.DrillFirstHoleInOperation

        If e.DrillingType = DrillingType.RigidTapping Then
            MessageBox.Show("Rigid tapping not supported on the Mach3 control", "Mach3", MessageBoxButtons.OK, MessageBoxIcon.Warning)
			Return
        End If

        NcCode.LineNumber("N")
        NcCode.NewDimensionedWord("X", e.PositionX.ToString(format, CultureInfo), Modal.On)
        NcCode.NewDimensionedWord("Y", e.PositionY.ToString(format, CultureInfo), Modal.On)
        NcCode.NewDimensionedWord("Z", (e.IntialRapid + toolLengthOffset).ToString(format, CultureInfo), Modal.Off)
        NcCode.NewLine()

		NcCode.LineNumber("N")
		Dim feedrate As Double
        If e.DrillingType = DrillingType.Drilling And e.Dwell = 0 Then
            NcCode.NewString("G81 ")
            feedrate = e.FeedRate
        ElseIf e.DrillingType = DrillingType.Drilling And e.Dwell > 0 Then
            NcCode.NewString("G82 ")
            feedrate = e.FeedRate
        ElseIf e.DrillingType = DrillingType.PeckDrill Then
            NcCode.NewString("G83 ")
            feedrate = e.FeedRate
        ElseIf e.DrillingType = DrillingType.ChipBreaker Then
            NcCode.NewString("G73 ")
            feedrate = e.FeedRate
        ElseIf e.DrillingType = DrillingType.Tapping Then
            NcCode.NewString("G84 ")
            feedrate = e.Pitch * e.SpindleSpeed * 0.95 'Reduce feedrate by 5% for floating holder
        ElseIf e.DrillingType = DrillingType.Boring Or e.DrillingType = DrillingType.Reaming Then
            NcCode.NewString("G85 ")
            feedrate = e.FeedRate
        ElseIf e.DrillingType = DrillingType.CustomCycle1 Then
			'Add code for additional drilling cycle
		End If
        Dim returnType As String
        If e.RetractToIntialRapid Then
            returnType = "G98"
        Else
            returnType = "G99"
        End If
        NcCode.NewString(returnType & " R" & (e.FeedFrom + toolLengthOffset).ToString(format, CultureInfo) & " ")
        NcCode.NewDimensionedWord("Z", (e.FinishDepth + toolLengthOffset).ToString(format, CultureInfo), Modal.Off)
        If (e.DrillingType = DrillingType.Drilling) And e.Dwell > 0 Then
            NcCode.NewString("P" & e.Dwell * 1000 & " ")
        ElseIf e.DrillingType = DrillingType.PeckDrill Or e.DrillingType = DrillingType.ChipBreaker Then
            NcCode.NewString("Q" & e.PeckAmount.ToString(format, CultureInfo) & " ")
        End If
        NcCode.NewDimensionedWord("F", feedrate.ToString(format, CultureInfo), Modal.Off)
        NcCode.NewLine()

    End Sub

    ''' <summary>
    ''' This event handler is invoked to drill a hole that is NOT after a Tool change or the first in the operation.
    ''' </summary>
    ''' <param name="e">Provides data for the drill hole event.</param>
    Private Sub DrillHole(ByVal e As DrillHoleEventArgs) Implements IMachineDefinition_2_1.DrillHole

        NcCode.LineNumber("N")
        NcCode.NewDimensionedWord("X", e.PositionX.ToString(format, CultureInfo), Modal.On)
        NcCode.NewDimensionedWord("Y", e.PositionY.ToString(format, CultureInfo), Modal.On)
        NcCode.NewLine()

    End Sub

    ''' <summary>
    ''' This event handler is invoked when a drilling cycle needs to be canelled.
    ''' </summary>
    ''' <param name="e">Provides data for the cancel drilling cycle event.</param>
    Private Sub CancelDrillingCycles(ByVal e As CancelDrillingCyclesEventArgs) Implements IMachineDefinition_2_1.CancelDrillingCycles

		NcCode.LineNumber("N")
        NcCode.NewString("G80")
        NcCode.NewLine()

    End Sub

    ''' <summary>
    ''' This event handler is invoked last.
    ''' </summary>
    Private Sub ProgramEnd(ByVal e As ProgramEndEventArgs) Implements IMachineDefinition_2_1.ProgramEnd

		Me.PositionForToolChange()
	
		If _configuration.YAxisUnload Then
			Dim unloadPosition As String
			If e.Metric
				unloadPosition =  _configuration.YAxisUnloadPosition.ToString(format, CultureInfo)
			Else
				unloadPosition = (_configuration.YAxisUnloadPosition / 25.4).ToString(format, CultureInfo)
			End If
			NcCode.NewLine()
			NcCode.LineNumber("N")
			NcCode.NewString("G90")
			NcCode.NewLine()
			NcCode.LineNumber("N")
			NcCode.NewString("G0 G53 Y" & unloadPosition)
		End If
		NcCode.NewLine()
		NcCode.LineNumber("N")
		NcCode.NewString("M30")
		NcCode.NewLine()

    End Sub

    ''' <summary>
    ''' This event handler is invoked so the start of the file can be created.
    ''' </summary>
    ''' <param name="e">Provides data for the program start event.</param>
    ''' <remarks>Here is where you should create the end of the file, for example the % sign as used by a Fanuc.</remarks>
    Private Sub FileEnd(ByVal e As FileEndEventArgs) Implements IMachineDefinition_2_1.FileEnd
        
		NcCode.NewString("%")
        NcCode.NewLine()

	End Sub

    ''' <summary>
    ''' This event handler is invoked to define the start of all Sub Routines.
    ''' </summary>
    ''' <param name="e">Provides data for the all sub routine start event.</param>
    Private Sub SubRoutineAllStart(ByVal e As SubRoutineAllStartEventArgs) Implements IMachineDefinition_2_1.SubRoutineAllStart


	End Sub

    ''' <summary>
    ''' This event handler is invoked when a Sub Routine is called.
    ''' </summary>
    ''' <param name="e">Provides data for the all sub routine start event.</param>
    Private Sub SubRoutineCall(ByVal e As SubRoutineCallEventArgs) Implements IMachineDefinition_2_1.SubRoutineCall

		NcCode.LineNumber("N")
        NcCode.NewString("M98 P" & e.SubRoutineNumber)
        NcCode.NewLine()

	End Sub

    ''' <summary>
    ''' This event handler is invoked to define the start of the Sub Routine.
    ''' </summary>
    ''' <param name="e">Provides data for the sub routine start event.</param>
    Private Sub SubRoutineStart(ByVal e As SubRoutineStartEventArgs) Implements IMachineDefinition_2_1.SubRoutineStart
        
		NcCode.NewString("O" & e.SubRoutineNumber)
        NcCode.NewLine()

	End Sub

    ''' <summary>
    ''' This event handler is invoked to define the end of the Sub Routine.
    ''' </summary>
    ''' <param name="e">Provides data for the sub routine end event.</param>
    Private Sub SubRoutineEnd(ByVal e As SubRoutineEndEventArgs) Implements IMachineDefinition_2_1.SubRoutineEnd

		NcCode.NewString("M99")
        NcCode.NewLine()

	End Sub

    ''' <summary>
    ''' This event handler is invoked to define the end of all Sub Routines.
    ''' </summary>
    ''' <param name="e">Provides data for the all sub routine end event.</param>
    Private Sub SubRoutineAllEnd(ByVal e As SubRoutineAllEndEventArgs) Implements IMachineDefinition_2_1.SubRoutineAllEnd


	End Sub

	''' <summary>
	''' <summary>
	''' This event handler is invoked so a program stop/comment can be created.
	''' </summary>
	''' <param name="e">Provides data for the program stop event.</param>
	''' <remarks>Here is where you should create all the logic for a program stop, typlically M00/M01.</remarks>
	Private Sub ProgramStop(ByVal e As ProgramStopEventArgs) Implements IMachineDefinition_2_1.ProgramStop

		If e.ProgramStopType = ProgramStopType.Stop Then
			NcCode.LineNumber("N")
			NcCode.NewString("M0")
			NcCode.NewLine()			
		Else If e.ProgramStopType = ProgramStopType.Optional Then
			NcCode.LineNumber("N")
			NcCode.NewString("M1")
			NcCode.NewLine()			
		End If
		For Each commentLine As String In  e.Comment
                NcCode.LineNumber("N")
				If commentLine.Length > 0 Then
                	NcCode.NewString("(" & commentLine.ToUpper() & ")")				
				End If
                NcCode.NewLine()
        Next

	End Sub

	''' <summary>
	''' This event handler is invoked at the beginning of an interpolation move, for Profiling Operations only.
	''' </summary>
	''' <param name="e">Provides data for the start interpolation event.</param>
	''' <remarks>Used for turning on the torch when using Jet cutting.</remarks>
	Private Sub StartInterpolation(ByVal e As StartInterpolationEventArgs) Implements IMachineDefinition_2_1.StartInterpolation

	End Sub

	''' <summary>
	''' This event handler is invoked at the end of an interpolation move, for Profiling Operations only.
	''' </summary>
	''' <param name="e">Provides data for the end interpolation event.</param>
	''' <remarks>Used for turning off the torch when using Jet cutting.</remarks>
	Private Sub EndInterpolation(ByVal e As EndInterpolationEventArgs) Implements IMachineDefinition_2_1.EndInterpolation

	End Sub

	''' <summary>
	''' Position the axes for a Tool Change
	''' </summary>
	Private Sub PositionForToolChange()

		If Not _configuration.ToolChangePosition = ToolChangePosition.DoNotOutput Then
			NcCode.LineNumber("N")
		End If
		If _configuration.ToolChangePosition = ToolChangePosition.G28G49 Then
 	       NcCode.NewString("G49 Z0")			
		   NcCode.NewLine()
	       NcCode.LineNumber("N")
 	       NcCode.NewString("G91 G28 M09")
		Else If _configuration.ToolChangePosition = ToolChangePosition.G28
 	       NcCode.NewString("G91 G28 Z0 M09")			
		Else If _configuration.ToolChangePosition = ToolChangePosition.G30
 	       NcCode.NewString("G91 G30 Z0 M09")
		Else If _configuration.ToolChangePosition = ToolChangePosition.CallSubProgram
 	       NcCode.NewString("M98P" & _configuration.SubProgramATC)
		Else If _configuration.ToolChangePosition = ToolChangePosition.G53
 	       NcCode.NewString("G0 G53 Z0")
		Else If _configuration.ToolChangePosition = ToolChangePosition.EnteredPosition Then
 	    	Dim xPosition, yPosition, zPosition As String
			xPosition = _configuration.XAxisToolUnloadPosition.ToString(format, CultureInfo)   
 	    	yPosition = _configuration.YAxisToolUnloadPosition.ToString(format, CultureInfo)   
 	    	zPosition = _configuration.ZAxisToolUnloadPosition.ToString(format, CultureInfo)   
 	        NcCode.NewString("G49 Z" & ZPosition)			
		    NcCode.NewLine()
	        NcCode.LineNumber("N")
		    NcCode.NewWord("G0",  Modal.On)
 	        NcCode.NewString(" X" & XPosition & " Y" & YPosition)
		End If	
		If Not _configuration.ToolChangePosition = ToolChangePosition.DoNotOutput Then
	        NcCode.NewLine()
		end if
		
	End Sub

#End Region

#Region "User Congigured Input"

    ''' <summary>
    ''' A String varable to store the program number, this is available through out the
    ''' Machine Definition
    ''' </summary>
    Private programNumber As String

    ''' <summary>
    ''' A String varable to store the fixture offset, this is available through out the
    ''' Machine Definition
    ''' </summary>
    Private fixtureOffset As String

    ''' <summary>
    ''' When the user clicks the 'User Configured Input' button on the NC Code tab in the Part Manager,
    ''' this sub procedure is called. Here we display the dialog box where the user can input information
	''' that can be used in the Machine Definition.
    ''' </summary>
    Private Sub UserConfiguredInput(ByVal e As UserConfiguredInputEventArgs) Implements IMachineDefinition_2_1.UserConfiguredInput

        UserConfiguredInputDialog.Display()
        programNumber = UserConfiguredInputDialog.GetInputBoxValue("Program Number")
        fixtureOffset = UserConfiguredInputDialog.GetInputBoxValue("Fixture Offset")
        NcCode.GenerateCode()

    End Sub

#End Region

#Region "Properties"

    ''' <summary>
    ''' Gets the Nc code to be output for a clockwise spindle start.
    ''' </summary>
    Private ReadOnly Property CWSpindleStart() As String Implements IMachineDefinition_2_1.CWSpindleStart
        Get
            Return "M3"
        End Get
    End Property

    ''' <summary>
    ''' Gets the Nc code to be output for a counter clockwise spindle start.
    ''' </summary>
    Private ReadOnly Property CCWSpindleStart() As String Implements IMachineDefinition_2_1.CCWSpindleStart
        Get
            Return "M4"
        End Get
    End Property

    ''' <summary>
    ''' Gets the Nc code to be output for flood coolant.
    ''' </summary>
    Private ReadOnly Property CoolantFlood() As String Implements IMachineDefinition_2_1.CoolantFlood
        Get
            Return "M8"
        End Get
    End Property

    ''' <summary>
    ''' Gets the Nc code to be output for coolant through spindle.
    ''' </summary>
    Private ReadOnly Property CoolantThroughSpindle() As String Implements IMachineDefinition_2_1.CoolantThroughSpindle
        Get
            Return ""
        End Get
    End Property

    ''' <summary>
    ''' Gets the Nc code to be output for coolant through the tool.
    ''' </summary>
    Private ReadOnly Property CoolantThroughTool() As String Implements IMachineDefinition_2_1.CoolantThroughTool
        Get
            Return ""
        End Get
    End Property

    ''' <summary>
    ''' Gets the Nc code to be output for mist coolant.
    ''' </summary>
    Private ReadOnly Property CoolantMist() As String Implements IMachineDefinition_2_1.CoolantMist
        Get
            Return "M7"
        End Get
    End Property

    ''' <summary>
    ''' Gets the Nc code to be output for an air blast.
    ''' </summary>
    Private ReadOnly Property CoolantAirBlast() As String Implements IMachineDefinition_2_1.CoolantAirBlast
        Get
            Return ""
        End Get
    End Property

    ''' <summary>
    ''' Gets the Nc code to be output for coolant off.
    ''' </summary>
    Private ReadOnly Property CoolantNone() As String Implements IMachineDefinition_2_1.CoolantNone
        Get
            Return "M9"
        End Get
    End Property

    ''' <summary>
    ''' Gets the Nc code to be output for custom coolant.
    ''' </summary>
    Private ReadOnly Property CoolantCustom() As String Implements IMachineDefinition_2_1.CoolantCustom
        Get
            Return ""
        End Get
    End Property

#End Region

#Region "Modals"

    ''' <summary>
    ''' This event handler is invoked to sets up modal words
    ''' </summary>
    ''' <param name="e">Provides data for the modals event</param>
    Private Sub Modals(ByVal e As ModalEventArgs) Implements IMachineDefinition_2_1.Modals

        'Firstly create a new 'ModalWordGroup'.
        Dim wordGroup1 As New ModalWordGroup()
        'Add Words to 'wordGroup1'.
        wordGroup1.Words.Add("G0")
        wordGroup1.Words.Add("G1")
        wordGroup1.Words.Add("G2")
        wordGroup1.Words.Add("G3")

        'Finally add the groups to 'WordGroups'.
        e.WordGroups.Add(wordGroup1)

        'Add Absolute Modal Dimentioned Words to 'AbsoluteDimensionedWords'.	
        e.AbsoluteDimensionedWords.Add(New ModalAbsoluteDimensionedWord("X"))
        e.AbsoluteDimensionedWords.Add(New ModalAbsoluteDimensionedWord("Y"))
        e.AbsoluteDimensionedWords.Add(New ModalAbsoluteDimensionedWord("Z"))
        e.AbsoluteDimensionedWords.Add(New ModalAbsoluteDimensionedWord("F"))

        'Add Incremental Modal Dimentioned Words to 'IncrementalDimensionedWords'.		
        e.IncrementalDimensionedWords.Add(New ModalIncrementalDimensionedWord("I"))
        e.IncrementalDimensionedWords.Add(New ModalIncrementalDimensionedWord("J"))

    End Sub

#End Region

End Class

#Region "Configuration"

''' <summary>
''' Class to manage the configuration of this Machine Definition.
''' </summary>
Public Class Configuration
	
	Private _toolChangerType As ToolChangerType
	Private _toolChangePosition As ToolChangePosition
	Private _commentsType As CommentsType
	Private _outputLineNumbers As Boolean
	Private _outputLineNumberAtATC As Boolean
	Private _outputM01ForATC As M01Type
	Private _subProgramATC As Integer
	Private _mCodeForAtc As Integer
	Private _arcCentre As ArcCentre
	Private _codeToApplyToolOffsets As CodeToApplyToolOffsets
	Private _yAxisUnload As Boolean
	Private _yAxisUnloadPosition As Double
	Private _xAxisToolUnloadPosition As Double
	Private _yAxisToolUnloadPosition As Double
	Private _zAxisToolUnloadPosition As Double
	Private _outputCycleTime As Boolean
	Private _chipToChipAtcTime As Double
	Private _rapidTraverseRate As Double
	Private _rapidPositionAccelerationDeceleration As Double
	Private _outputFixtureOffset As Boolean
	Private _addOffsetValue As Boolean
	
	''' <summary>
	''' Gets or sets the type of tool changer.
	''' </summary>
	<System.ComponentModel.Description("NoPreSelectType = T code and M6 on the same line. PreSelectType = The tool changer pre selects tools. UseMacroMCode = Use a macro M code to perform the tool change"), _
	System.ComponentModel.DisplayName("Type of Tool changer"), _
	System.ComponentModel.Category("Tool Change")> _
	Public Property ToolChangerType() As ToolChangerType
        Get
            Return _toolChangerType
        End Get
        Set
            _toolChangerType = value
        End Set
    End Property
	
	''' <summary>
	''' Gets or sets the tool changer position.
	''' </summary>
	<System.ComponentModel.Description("The position where the tool change takes place."), _
	System.ComponentModel.DisplayName("Tool Change position"), _
	System.ComponentModel.Category("Tool Change")> _
	Public Property ToolChangePosition() As ToolChangePosition
        Get
            Return _toolChangePosition
        End Get
        Set
            _toolChangePosition = value
        End Set
    End Property
	
	''' <summary>
	''' Gets or sets a value indicating the type of comments to use.
	''' </summary>
	<System.ComponentModel.Description("The type of comments to use"), _
	System.ComponentModel.DisplayName("Comments Type")> _
	Public Property CommentsType() As CommentsType
        Get
            Return _commentsType
        End Get
        Set
            _commentsType = value
        End Set
    End Property
	
	''' <summary>
	''' Gets or sets a value indicating if line numbers should be output.
	''' </summary>
	<System.ComponentModel.Description("Should line numbers be output?"), _
	System.ComponentModel.DisplayName("Output Line Numbers")> _
	Public Property OutputLineNumbers() As Boolean
        Get
            Return _outputLineNumbers
        End Get
        Set
            _outputLineNumbers = value
        End Set
    End Property

	''' <summary>
	''' Gets or sets a value indicating if a line number the sane at the tool number should be output at the ATC line.
	''' </summary>
	<System.ComponentModel.Description("Should a line number be output the same as the tool number on the ATC line?"), _
	System.ComponentModel.DisplayName("Output Line Number at ATC"), _
	System.ComponentModel.Category("Tool Change")> _
	Public Property OutputLineNumberAtATC() As Boolean
        Get
            Return _outputLineNumberAtATC
        End Get
        Set
            _outputLineNumberAtATC = value
        End Set
    End Property	
	
	''' <summary>
	''' Gets or sets a value indicating if and where an M01 is output for an ATC.
	''' </summary>
	<System.ComponentModel.Description("Specifies if and where an M01 is output for an ATC"), _
	System.ComponentModel.DisplayName("Output M01 for ATC"), _
	System.ComponentModel.Category("Tool Change")> _
	Public Property OutputM01ForATC() As M01Type
        Get
            Return _outputM01ForATC
        End Get
        Set
            _outputM01ForATC = value
        End Set
    End Property
	
	''' <summary>
	''' Gets or sets a value indicating the sub program number to be called for the ATC position
	''' </summary>
	<System.ComponentModel.Description("The sub program number to be called for the ATC position"), _
	System.ComponentModel.DisplayName("Sub program number for ATC position"), _
	System.ComponentModel.Category("Tool Change")> _
	Public Property SubProgramATC() As Integer
        Get
            Return _subProgramATC
        End Get
        Set
            _subProgramATC = value
        End Set
    End Property
	
	''' <summary>
	''' Gets or sets a value indicating the sub program number to be called for the macro M code tool change
	''' </summary>
	<System.ComponentModel.Description("The sub program number to be called for the macro M code tool change"), _
	System.ComponentModel.DisplayName("Sub program number for macro M code"), _
	System.ComponentModel.Category("Tool Change")> _
	Public Property MCodeForAtc() As Integer
        Get
            Return _mCodeForAtc
        End Get
        Set
            _mCodeForAtc = value
        End Set
    End Property
	
	''' <summary>
	''' Gets or sets a value indicating the type of arc centre to use
	''' </summary>
	<System.ComponentModel.Description("The type of arc centre to use"), _
	System.ComponentModel.DisplayName("Arc centre")> _
	Public Property ArcCentre() As ArcCentre
        Get
            Return _arcCentre
        End Get
        Set
            _arcCentre = value
        End Set
    End Property
	
	''' <summary>
	''' Gets or sets a value indicating the G code used to apply tool length offsets
	''' </summary>
	<System.ComponentModel.Description("The G code used to apply tool length offsets"), _
	System.ComponentModel.DisplayName("Tool length G code")> _
	Public Property CodeToApplyToolOffsets() As CodeToApplyToolOffsets
        Get
            Return _codeToApplyToolOffsets
        End Get
        Set
            _codeToApplyToolOffsets = value
        End Set
    End Property
	
	''' <summary>
	''' Gets or sets the position from the reference point that the Y axis is positioned at the end of the program.
	''' </summary>
	<System.ComponentModel.Description("The position in METRIC from the reference point that the Y axis is positioned at the end of the program."), _
	System.ComponentModel.DisplayName("Y axis unload position"),  _
	System.ComponentModel.Category("Unload")> _
	Public Property YAxisUnloadPosition() As Double
        Get
            Return _yAxisUnloadPosition
        End Get
        Set
            _yAxisUnloadPosition = value
        End Set
    End Property
	
	''' <summary>
	''' Gets or sets a value indicating if the Y axis be moved to an unload position at the end of the program.
	''' </summary>
	<System.ComponentModel.Description("Should the Y axis be moved to an unload position at the end of the program?"), _
	System.ComponentModel.DisplayName("Y axis unload"),  _
	System.ComponentModel.Category("Unload")> _
	Public Property YAxisUnload() As Boolean
        Get
            Return _yAxisUnload
        End Get
        Set
            _yAxisUnload = value
        End Set
    End Property

	''' <summary>
	''' Gets or sets the position the Y axis is positioned for a tool change.
	''' </summary>
	<System.ComponentModel.Description("The position from the work reference point that the X axis is positioned for a tool change."), _
	System.ComponentModel.DisplayName("X axis tool change position"),  _
	System.ComponentModel.Category("Tool Change")> _
	Public Property XAxisToolUnloadPosition() As Double
        Get
            Return _xAxisToolUnloadPosition
        End Get
        Set
            _xAxisToolUnloadPosition = value
        End Set
    End Property

	<System.ComponentModel.Description("The position from the work reference point that the Y axis is positioned for a tool change."), _
	System.ComponentModel.DisplayName("Y axis tool change position"),  _
	System.ComponentModel.Category("Tool Change")> _
	Public Property YAxisToolUnloadPosition() As Double
        Get
            Return _yAxisToolUnloadPosition
        End Get
        Set
            _yAxisToolUnloadPosition = value
        End Set
    End Property

	<System.ComponentModel.Description("The position from the work reference point that the Z axis is positioned for a tool change."), _
	System.ComponentModel.DisplayName("Z axis tool change position"),  _
	System.ComponentModel.Category("Tool Change")> _
	Public Property ZAxisToolUnloadPosition() As Double
        Get
            Return _zAxisToolUnloadPosition
        End Get
        Set
            _zAxisToolUnloadPosition = value
        End Set
    End Property


	''' <summary>
	''' Gets or sets a value indicating if the cycle time should be output in a comment.
	''' </summary>
	<System.ComponentModel.Description("Should the cycle time be output?"), _
	System.ComponentModel.DisplayName("Output Cycle Time"), _
	System.ComponentModel.Category("Cycle Time")> _
	Public Property OutputCycleTime() As Boolean
        Get
            Return _outputCycleTime
        End Get
        Set
            _outputCycleTime = value
        End Set
    End Property
	
	''' <summary>
	''' Gets or sets the chip to chip tool change time in seconds.
	''' </summary>
	<System.ComponentModel.Description("The chip to chip tool change time in seconds"), _
	System.ComponentModel.DisplayName("Chip to chip tool change time"), _
	System.ComponentModel.Category("Cycle Time")> _
	Public Property ChipToChipAtcTime() As Double
        Get
            Return _chipToChipAtcTime
        End Get
        Set
            _chipToChipAtcTime = value
        End Set
    End Property
	
	''' <summary>
	''' Gets or sets the rapid traverse rate in millimetres/minute.
	''' </summary>
	<System.ComponentModel.Description("The rapid traverse rate in millimetres/minute. This value is converted to imperial for Parts with Inch units."), _
	System.ComponentModel.DisplayName("Rapid traverse rate"), _
	System.ComponentModel.Category("Cycle Time")> _
	Public Property RapidTraverseRate() As Double
        Get
            Return _rapidTraverseRate
        End Get
        Set
            _rapidTraverseRate = value
        End Set
    End Property
	
	''' <summary>
	''' Gets or sets the time for a each Rapid move that should be added to the cycle time due to Acceleration/Deceleration.
	''' </summary>
	<System.ComponentModel.Description("The time for a each Rapid move that should be added to the cycle time due to Acceleration/Deceleration"), _
	System.ComponentModel.DisplayName("Rapid Acceleration/Deceleration"), _
	System.ComponentModel.Category("Cycle Time")> _
	Public Property RapidPositionAccelerationDeceleration() As Double
        Get
            Return _rapidPositionAccelerationDeceleration

        End Get
        Set
            _rapidPositionAccelerationDeceleration = value
        End Set
    End Property
	
	''' <summary>
	''' Gets or sets a value indicating if fixture offsets should be output.
	''' </summary>
	<System.ComponentModel.Description("Should fixture offsets should be output?"), _
	System.ComponentModel.DisplayName("Output Fixture Offsets")> _
	Public Property OutputFixtureOffset() As Boolean
        Get
            Return _outputFixtureOffset

        End Get
        Set
            _outputFixtureOffset = value
        End Set
    End Property
	
	''' <summary>
	''' Gets or sets a value indicating if the Offset value in the Tool Manager for each tool is added to the Z positions
	''' </summary>
	<System.ComponentModel.Description("Should the Tool Offset value be added to Z positions?"), _
	System.ComponentModel.DisplayName("Use Tool Offset value")> _
	Public Property AddOffsetValue() As Boolean
        Get
            Return _addOffsetValue

        End Get
        Set
            _addOffsetValue = value
        End Set
    End Property

End Class

''' <summary>
''' The type of tool changer.
''' </summary>
Public Enum ToolChangerType

	''' <summary>
	''' T code and M6 on the same line.
	''' </summary>
	NoPreSelectType
	
	''' <summary>
	''' The tool changer pre select tools.
	''' </summary>
	PreSelectType
	
	''' <summary>
	''' Use a macro M code to perform the tool change
	''' </summary>
	UseMacroMCode

	
End Enum

''' <summary>
''' The tool changer position.
''' </summary>
Public Enum ToolChangePosition
	
	''' <summary>
	''' Do not output
	''' </summary>
	DoNotOutput

	''' <summary>
	''' The tool change is at the first reference point.
	''' </summary>
	G28
	
	''' <summary>
	''' The tool change is at the first reference point and turns off tool offsets.
	''' </summary>
	G28G49

	''' <summary>
	''' The tool change is at the second reference point.
	''' </summary>
	G30
			
	''' <summary>
	''' The tool change is at G53 Z0.
	''' </summary>
	G53
	
	''' <summary>
	''' The tool changer position is determined by a sub program.
	''' </summary>
	CallSubProgram
	
	''' <summary>
	''' The tool changer position is determined by the values entered.
	''' </summary>
	EnteredPosition

End Enum

''' <summary>
''' The position of M01.
''' </summary>
Public Enum M01Type

	''' <summary>
	''' Do not output M01.
	''' </summary>
	DoNotOutput

	''' <summary>
	''' Output M01 before the ATC.
	''' </summary>
	OutputBeforeATC
	
	''' <summary>
	''' Output M01 after the ATC.
	''' </summary>
	OutputAfterATC
	
End Enum

''' <summary>
''' The type of comments.
''' </summary>
Public Enum CommentsType

	''' <summary>
	''' Use open and close brackets.
	''' </summary>
	Parenthesis

	''' <summary>
	''' Use two forward slashes.
	''' </summary>
	DoubleForwardSlash
	
End Enum

''' <summary>
''' The type of arc centre.
''' </summary>
Public Enum ArcCentre

	''' <summary>
	''' Use incremental I and J.
	''' </summary>
	Incremental

	''' <summary>
	''' Use absolute I and J.
	''' </summary>
	Absolute
		
End Enum

''' <summary>
''' The G code used to apply tool length offset.
''' </summary>
Public Enum CodeToApplyToolOffsets

	''' <summary>
	''' Use G43.
	''' </summary>
	G43

	''' <summary>
	''' Use G44.
	''' </summary>
	G44
	
End Enum

#End Region